@@ -32,7 +32,7 @@ void meta_workspace_queue_calc_showing   (MetaWorkspace *workspace);
 static void set_active_space_hint        (MetaScreen *screen);
 static void focus_ancestor_or_mru_window (MetaWorkspace *workspace,
                                           MetaWindow    *not_this_one,
-                                          Time           timestamp);
+                                          guint32        timestamp);
 
 static void
 maybe_add_to_list (MetaScreen *screen, MetaWindow *window, gpointer data)
@@ -68,6 +68,7 @@ meta_workspace_new (MetaScreen *screen)
   workspace->xinerama_region = NULL;
   workspace->screen_edges = NULL;
   workspace->xinerama_edges = NULL;
+  workspace->list_containing_self = g_list_prepend (NULL, workspace);
 
   workspace->all_struts = NULL;
 
@@ -114,6 +115,7 @@ meta_workspace_free (MetaWorkspace *workspace)
 
   g_list_free (workspace->mru_list);
   g_slist_free (workspace->all_struts);
+  g_list_free (workspace->list_containing_self);
 
   for (i = 0; i < screen->n_xinerama_infos; i++)
     meta_rectangle_free_list_and_elements (workspace->xinerama_region[i]);
@@ -272,7 +274,7 @@ meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
 void
 meta_workspace_activate_with_focus (MetaWorkspace *workspace,
                                     MetaWindow    *focus_this,
-                                    Time           timestamp)
+                                    guint32        timestamp)
 {
   MetaWorkspace *old;
   MetaWindow *move_window;
@@ -348,7 +350,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
 
 void
 meta_workspace_activate (MetaWorkspace *workspace,
-                         Time           timestamp)
+                         guint32        timestamp)
 {
   meta_workspace_activate_with_focus (workspace, NULL, timestamp);
 }
@@ -421,7 +423,7 @@ set_active_space_hint (MetaScreen *screen)
   
   data[0] = meta_workspace_index (screen->active_workspace);
 
-  meta_verbose ("Setting _NET_CURRENT_DESKTOP to %ld\n", data[0]);
+  meta_verbose ("Setting _NET_CURRENT_DESKTOP to %lu\n", data[0]);
   
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -870,7 +872,7 @@ meta_workspace_get_name (MetaWorkspace *workspace)
 void
 meta_workspace_focus_default_window (MetaWorkspace *workspace,
                                      MetaWindow    *not_this_one,
-                                     Time           timestamp)
+                                     guint32        timestamp)
 {
   if (timestamp == CurrentTime)
     {
@@ -947,9 +949,10 @@ record_ancestor (MetaWindow *window,
 static void
 focus_ancestor_or_mru_window (MetaWorkspace *workspace,
                               MetaWindow    *not_this_one,
-                              Time           timestamp)
+                              guint32        timestamp)
 {
   MetaWindow *window = NULL;
+  MetaWindow *desktop_window = NULL;
   GList *tmp;
 
   if (not_this_one)
@@ -994,12 +997,24 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
           tmp_window->type != META_WINDOW_DESKTOP)
         {
           window = tmp->data;
-	  break;
+          break;
+        }
+      else if (tmp_window != not_this_one      &&
+               desktop_window == NULL          &&
+               meta_window_showing_on_its_workspace (tmp_window) &&
+               tmp_window->type == META_WINDOW_DESKTOP)
+        {
+          /* Found the most recently used desktop window */
+          desktop_window = tmp_window;
         }
 
       tmp = tmp->next;
     }
 
+  /* If no window was found, default to the MRU desktop-window */
+  if (window == NULL)
+    window = desktop_window;
+
   if (window)
     {
       meta_topic (META_DEBUG_FOCUS,
